'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function () {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$defaultLogfile = _ref.defaultLogfile,
      defaultLogfile = _ref$defaultLogfile === undefined ? 'default' : _ref$defaultLogfile,
      _ref$level = _ref.level,
      level = _ref$level === undefined ? 'info' : _ref$level;

  var logPath = _path2.default.join(process.cwd(), '/logs');

  // check that it exists and create it if it don't
  try {
    _fs2.default.statSync(logPath);
  } catch (err) {
    _fs2.default.mkdirSync(logPath);
  }

  var defaultLogger = new _winston2.default.Logger({
    transports: [new _winston2.default.transports.File({
      filename: _path2.default.join(logPath, defaultLogfile + '.log'),
      level: level
    })]
  });

  var loggers = { default: defaultLogger };

  var addLogger = function addLogger(logFile) {
    var newLogger = new _winston2.default.Logger({
      transports: [new _winston2.default.transports.File({
        filename: _path2.default.join(logPath, logFile + '.log'),
        level: level
      })]
    });
    loggers[logFile] = newLogger;
  };

  var logToOther = function logToOther(logFile, msg, extra) {
    if (loggers[logFile]) {
      loggers[logFile].log(level, msg, extra);
    } else {
      // add logger and retry
      addLogger(logFile);
      logToOther(logFile, msg, extra);
    }
  };

  return {
    log: function log(msg, logfileOrExtra, extra) {
      if (typeof logfileOrExtra === 'string') {
        // we want to log to different logFile
        logToOther(logfileOrExtra, msg, extra);
      } else {
        defaultLogger.log(level, msg, logfileOrExtra);
      }
    }
  };
};

var _winston = require('winston');

var _winston2 = _interopRequireDefault(_winston);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }