'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createServer;

var _httpServer = require('./http-server');

var _httpServer2 = _interopRequireDefault(_httpServer);

var _socketServer = require('./socket-server');

var _socketServer2 = _interopRequireDefault(_socketServer);

var _serverStageManager = require('./server-stage-manager');

var _serverStageManager2 = _interopRequireDefault(_serverStageManager);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _network = require('./network');

var Network = _interopRequireWildcard(_network);

var _util = require('../util');

var Util = _interopRequireWildcard(_util);

var _commands = require('./commands');

var _events = require('./events');

var _events2 = require('../events');

var Events = _interopRequireWildcard(_events2);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function createServer() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$network = _ref.network,
      network = _ref$network === undefined ? Network.pairs(16) : _ref$network,
      _ref$logger = _ref.logger,
      logger = _ref$logger === undefined ? (0, _logger2.default)({}) : _ref$logger,
      _ref$events = _ref.events,
      events = _ref$events === undefined ? {} : _ref$events,
      _ref$commands = _ref.commands,
      commands = _ref$commands === undefined ? {} : _ref$commands,
      _ref$adminCommands = _ref.adminCommands,
      adminCommands = _ref$adminCommands === undefined ? {} : _ref$adminCommands,
      _ref$stages = _ref.stages,
      stages = _ref$stages === undefined ? [] : _ref$stages,
      _ref$options = _ref.options;

  _ref$options = _ref$options === undefined ? {} : _ref$options;
  var _ref$options$port = _ref$options.port,
      port = _ref$options$port === undefined ? 3000 : _ref$options$port,
      _ref$options$resumeCu = _ref$options.resumeCurrentStage,
      resumeCurrentStage = _ref$options$resumeCu === undefined ? false : _ref$options$resumeCu,
      _ref$options$clientPa = _ref$options.clientPassword,
      clientPassword = _ref$options$clientPa === undefined ? '' : _ref$options$clientPa,
      _ref$options$adminPas = _ref$options.adminPassword,
      adminPassword = _ref$options$adminPas === undefined ? '' : _ref$options$adminPas;

  var stageManager = void 0;
  var nameMap = {};

  var httpServer = (0, _httpServer2.default)({
    port: port,
    clientPassword: clientPassword,
    adminPassword: adminPassword
  });
  var socketServer = (0, _socketServer2.default)(httpServer.getIO());

  // Log/Messaging
  function log(msg, fileOrExtra, extra) {
    logger.log(msg, fileOrExtra, extra);
  }

  function send(topic, message) {
    var event = { type: topic, payload: message };
    return {
      toAll: function toAll() {
        socketServer.sendEvent(event).toAll();
      },
      toClient: function toClient(clientId) {
        socketServer.sendEvent(event).toClients([clientId]);
      },
      toNeighboursOf: function toNeighboursOf(clientId) {
        socketServer.sendEvent(event).toClients([clientId].concat(network.getNeighbours(clientId)));
      },
      toNeighboursOfExclusive: function toNeighboursOfExclusive(clientId) {
        socketServer.sendEvent(event).toClients(network.getNeighbours(clientId));
      },
      toClients: function toClients() {
        var clients = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

        socketServer.sendEvent(event).toClients(clients);
      },
      toAdmin: function toAdmin() {
        socketServer.sendEvent(event).toAdmin();
      }
    };
  }

  // Running
  function run() {
    stageManager = (0, _serverStageManager2.default)({
      stages: stages,
      getContext: function getContext() {
        return monsterr;
      },
      getPlayers: function getPlayers() {
        return network.getPlayers();
      },
      onStageStarted: function onStageStarted(stage) {
        return handleEvent({ type: Events.START_STAGE, payload: stage });
      },
      onStageEnded: function onStageEnded(stage) {
        return handleEvent({ type: Events.END_STAGE, payload: stage });
      },
      onGameOver: function onGameOver() {
        return handleEvent({ type: Events.GAME_OVER });
      }
    });
  }

  // Event handling
  function handleEvent(event) {
    Util.handleEvent(event, [_events.builtinEvents, events, stageManager.getEvents()], monsterr);
  }

  function handleCommand(cmd) {
    Util.handleCommand(cmd, [!cmd.clientId ? adminCommands : {}, !cmd.clientId ? _commands.builtinAdminCommands : {}, commands, stageManager.getCommands()], monsterr);
  }

  // Add/Remove players
  var addPlayer = function addPlayer(player) {
    if (network.getPlayers(player).indexOf(player) === -1) {
      network.addPlayer(player);
    }

    // possibly resume current stage
    var currentStage = stageManager.getCurrentStage();
    if (resumeCurrentStage && currentStage && currentStage.number !== -1) {
      setTimeout(function () {
        return monsterr.send(Events.START_STAGE, currentStage.number).toClient(player);
      }, 200);
    }
  };
  var removePlayer = function removePlayer(player) {
    network.removePlayer(player);
    stageManager.playerDisconnected(player);
  };

  // Naming
  var setName = function setName(id, name) {
    var prevName = monsterr.getName(id);
    nameMap[id] = name;
    monsterr.send(Events.SET_NAME, { id: id, name: name, prevName: prevName }).toClient(id);
    monsterr.send(Events.RENAME, { id: id, name: name, prevName: prevName }).toNeighboursOf(id);
  };
  var getName = function getName(id) {
    return nameMap[id] || id;
  };

  // Simple util to handle different connection
  // events in the same way
  var connectionHandler = function connectionHandler(event) {
    return [event, function (player) {
      return handleEvent({
        type: event,
        payload: player,
        clientId: player
      });
    }];
  };

  // Wiring up socketServer events
  socketServer.on('cmd', handleCommand);
  socketServer.on('event', handleEvent);
  socketServer.on.apply(socketServer, _toConsumableArray(connectionHandler(Events.CLIENT_CONNECTED)));
  socketServer.on.apply(socketServer, _toConsumableArray(connectionHandler(Events.CLIENT_RECONNECTED)));
  socketServer.on.apply(socketServer, _toConsumableArray(connectionHandler(Events.CLIENT_DISCONNECTED)));

  /** API */
  var monsterr = {
    // Basic
    run: run,
    send: send,
    log: log,

    // getNetwork: () => network,

    // Players
    setName: setName,
    getName: getName,
    addPlayer: addPlayer,
    removePlayer: removePlayer,
    getPlayers: function getPlayers() {
      return network.getPlayers();
    },
    getLatencies: function getLatencies() {
      return socketServer.getLatencies();
    },

    // Stages
    start: function start() {
      return stageManager.start();
    },
    nextStage: function nextStage() {
      return stageManager.next();
    },
    reset: function reset() {
      return stageManager.reset();
    },
    getCurrentStage: function getCurrentStage() {
      return stageManager.getCurrentStage();
    },
    playerFinishedStage: function playerFinishedStage(player, stageNo) {
      return stageManager.playerFinishedStage(player, stageNo);
    }
  };

  return monsterr;
}