'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handleEvent = handleEvent;
exports.handleCommand = handleCommand;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var isNode = typeof module.exports !== 'undefined' && typeof window === 'undefined';

/**
 * Given an event and an array of handler collections,
 * calls every matching handler with the specified context.
 * @param {*} event the event
 * @param {*} handlers the handler collections
 * @param {*} context the context
 */
function handleEvent() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      type = _ref.type,
      payload = _ref.payload,
      clientId = _ref.clientId;

  var handlers = arguments[1];
  var context = arguments[2];

  console.log('EVENT:', { type: type, payload: payload });
  handlers.forEach(function (handler) {
    return handler[type] && (isNode ? handler[type](context, clientId, payload) : handler[type](context, payload));
  });
}

/**
 * Given a command and an array of handler collections,
 * calls every matching handler with the specified context.
 * @param {*} command the command
 * @param {*} handlers the handler collections
 * @param {*} context the context
 */
function handleCommand() {
  var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      type = _ref2.type,
      _ref2$args = _ref2.args,
      args = _ref2$args === undefined ? [] : _ref2$args,
      clientId = _ref2.clientId;

  var handlers = arguments[1];
  var context = arguments[2];

  console.log('CMD:', { type: type, args: args, clientId: clientId });
  return handlers.reduce(function (shouldCall, handler) {
    var res = handler[type] && (isNode ? handler[type].apply(handler, [context, clientId].concat(_toConsumableArray(args))) : handler[type].apply(handler, [context].concat(_toConsumableArray(args))));

    return res === false ? false : shouldCall;
  }, true);
}